;  TASM version Hans Otten, Nov 2024 
;
;		   EQUATES and DATA STORAGE 
;          PATTERN GENERATOR FOR GRAY SCALE SETUP OF TWO OR MORE K-1008
;          VISIBLE MEMORY BOARDS.
;          THE FIRST PATTERN GENERATOR PRODUCES A GRAY SCALE CHECKER- 
;          BOARD PATTERN.   ALL SQUARES ARE THE SAME SIZE BUT THE SIZE IN
;          EACH PASS IS DETERMINED BY A RANDOM NUMBER GENERATOR.       THE
;          SQUARES GET BRIGHTER FROM LEFT TO RIGHT AND TOP TO BOTTOM.
;          THIS IS A PATTERN GENERATOR ROUTINE ONLY, NO
;          MEMORY FUNCTION TEST IF PERFORMED.

;          KIM SYSTEM EQUATES

KIMMON     =    	$1C22   	; ADDRESS OF SAVE MACHINE STATE ENTRY POINT
VMSIZ      =    	8192        ; SIZE OF EACH VISABLE MEMORY BOARD

;          BASE PAGE DATA STORAGE
           .ORG	0

;          MAIN PROGRAM DATA STORAGE

TlITCT    .BYTE 	0           ; ITERATION COUNT FOR PATTERN 1
VMORG     .WORD  $6000       	; ADDRESS OF FIRST VM BOARD (MOST
                                ; SIGNIFICANT)
VMNO      .BYTE  4           	; COUNT OF VISIBLE MEMORIES CONNECTED
                                ; TOGETHER



;          DATA STORAGE FOR CHECKERBOARD PATTERN

VMADR     .WORD	0           	; ADDRESS POINTER FOR VM DATA MANIPULATION
VMDATA    .BYTE 0           	; DATA DESTINED FOR VM
CKXSZ     .BYTE 0           	; X SIZE (WIDTH) OF CHECKER RECTANGLE
CKYSZ     .BYTE 0           	; Y SIZE (HEIGHT) OF CHECKER RECTANGLE
CKDTA     .BYTE 0           	; COLOR OF UPPER LEFT CHECKER RECTANGLE
CKDTAX    .BYTE 0           	; WORK COLOR DURING HORIZONTAL SCAN
CKDTAY    .BYTE 0           	; WORK COLOR DURING VERTICAL SCAN
CKYCT     .BYTE 0           	; COUNT OF CHECKER HEIGHT DURING VERTICAL
								; SCAN
HBYTCT    .BYTE 0           	; BYTE COUNT DURING HORIZONTAL SCAN
VMCNT     .BYTE 0           	; COUNT OF VM BOARDS PROCESSED
VMPGCT    .BYTE 0           	; COUNT OF VM PAGES PROCESSED
RANDNO    .WORD 1234        	; RANDOM NUMBER REGISTER
;		  MAIN PATTERN GENERATION PROGRAM
          .ORG 	$200         	; START PROGRAM CODE AT 200

MTEST     LDA	$E0         	; INITIALIZE STACK POINTER
          TXS
          CLD                 	; INSURE BINARY ARITHMETIC

;         PATTERN 1, 16 GRAYSCALE CHECKERBOARD PATTERS.

MAIN      LDA   #16          	; SET 16 ITERATION COUNT
          STA 	TlITCT
MAIN1     JSR   RNDEXP       	; GET AN EXPONENTIALLY DISTRIBUTED RANDOM
								; NUMBER IN A
          JSR   CKSZAD       	; ADJUST ACCORDING TO NUMBER OF VM'S
          BEQ   MAIN1        	; MUST NOT BE ZERO
          STA   CKXSZ        	; MAKE IT THE X CHECKER SIZE
          JSR   RNDEXP       	; GET ANOTHER
          JSR   CKSZAD       	; ADJUST IT
          BEQ   MAIN1        	; MUST NOT BE ZERO
          STA   CKYSZ        	; MAKE IT THE Y CHECKER SIZE
          LDA  	VMNO         	; INITIALIZE VM BOARD COUNT
          STA   VMCNT
          LDA   VMORG        	; INITIALIZE VMADDR TO ADDRESS OF MOST
          STA   VMADR        	; SIGNIFICANT VM
          LDA   VMORG+1
          STA   VMADR+1
MAIN2     LDA   #0           	; SET INITIAL COLOR TO BLACK
          STA   CKDTA
          JSR   CKGEN        	; GENERATE A CHECKERBOARD
          LSR   CKXSZ        	; PREPARE FOR NEXT MOST SIGNIFICANT
          LSR   CKYSZ        	; CHECKERBOARD, DIVIDE CKXSZ AND CKYSZ BY 2
          DEC   VMCNT        	; TEST IF ALL VM BOARDS DONE
          BNE   MAIN2        	; GO GENERATE SQUARES ON NEXT MOST SIG VM
          LDX   #0           	; WAIT AWHILE
MAIN3     LDY   #0
MAIN4     LDA   #4
MAIN5     CLC
          ADC   #-1
          BNE   MAIN5
          DEY
          BNE   MAIN4
          DEX
          BNE   MAIN3
          DEC   TlITCT       	; DECREMENT AND CHECK 16 ITERATION COUNTER
          BNE   MAIN1        	; LOOP UNTIL 16 ITERATIONS D0NE
          JMP   MAIN         	; CONTINUE

;       CHECKERBOARD PATTERN GENERATOR ROUTINES
;       STARTS AT UPPER LEFT CORNER OF SCREEN AND GENERATES A CHECKER-
;       BOARD PATTERN.
;       ENTER WITH CKXSZ SET TO CHECKER SQUARE WIDTH AND CKYSZ SET TO
;       CHECKER SQUARE HEIGHT AND CKDTA SET TO 0 FOR A BLACK UPPER LEFT
;       SQUARE OR SET TO $FF FOR A WHITE UPPER LEFT SQUARE.
;       USES ALL REGISTERS, PRESERVES CKXSZ, CKYSZ, CKDTA

CKGEN    LDA    #VMSIZ/256   	; SET NUMBER OF VM PAGES TO FILL
         STA    VMPGCT
         LDY    #8           	; INITIALIZE BIT COUNT
         LDA    CKDTA        	; COPY CKDTA TO VERTICAL WORK LOCATION
         STA    CKDTAY

;       START A ROW OF CHECKER BLOCKS

CKGNV    LDA    CKYSZ        	; SET Y SIZE IN CKYCT
         STA    CKYCT
		 
;        START A HORIZONTAL SCAN

CKGNH    LDA    CKDTAY       	; COPY VERTICAL CKDTA TO HORIZONTAL WORK
         STA    CKDTAX       	; LOCATION
         LDA    #40          	; INITIALIZE COUNT OF BYTES GENERATED IN
         STA    HBYTCT       	; A HORIZONTAL SCAN

CKGNH1   LDX    CKXSZ        	; SET X SIZE IN INDEX X
CKGNH2   LDA    CKDTAX       	; GENERATE A D0T = TO CURRENT VALUE OF
         ROL	A            	; CKDTAX
         ROL    VMDATA
         DEY                 	; COUNT D0TS GENERATED
         BNE    CKGNH4       	; SKIP AHEAD IF NOT 8 YET
         LDA    VMDATA       	; STORE A COMPLETED BYTE IN VM
         STA    (VMADR),Y
         INC    VMADR        	; INCREMENT VM ADDRESS
         BNE    CKGNH3
         INC    VMADR+1
         DEC    VMPGCT       	; DECREMENT VM PAGE COUNT TO SEE IF D0NE
         BEQ    CKGENF       	; JUMP OUT IF SO
CKGNH3   LDY    #8           	; RESTORE 8 BIT COUNT
         DEC    HBYTCT       	; TEST IF FINISHED WITH A HORIZONTAL SCAN
         BEQ    CKGNV1       	; JUMP IF SO
CKGNH4   DEX                 	; DECREMENT SQUARE WIDTH COUNT
         BNE    CKGNH2       	; GO GENERATE NEXT D0T
         LDA    CKDTAX       	; AT SQUARE BOUNDARY, FLIP COLOR
         EOR    #$FF
         STA    CKDTAX
         JMP    CKGNH1       	; GO GENERATE NEXT D0T

;       FINISH VERTICAL SCAN

CKGNV1   DEC    CKYCT        	; DECREMENT SQUARE HEIGHT
         BNE    CKGNH        	; GO GENERATE NEXT LINE
         LDA    CKDTAY       	; AT SQUARE BOUNDARY, FLIP COLOR
		 EOR 	#$FF
		 STA 	CKDTAY
		 JMP 	CKGNV  			; GO GENERATE NEXT LINE

CKGENF 	 RTS    				; RETURN
;		MISCELLANEOUS ROUTINES 
;       RANDOM NUMBER GENERATOR SUBROUTINE
;       ENTER WITH SEED IN RANDNO
;       EXIT WITH NEW RANDOM NUMBER IN RANDNO
;       USES 16 BIT FEEDBACK SHIFT REGISTER METHOD 
;       DESTROYS REGISTER A AND Y
 
RAND   	LDY   	#8         		; SET COUNTER FOR 8 RANDOM BITS
RAND1  	LDA   	RANDNO     		; EXCLUSIVE-OR BITS 3, 12, 14, AND 15
		LSR   	A          		; OF SEED
		EOR   	RANDNO
		LSR   	A
		LSR   	A
		EOR   	RANDNO
		LSR   	A
        EOR   	RANDNO+1   		; RESULT IS IN BIT 3 OF A
		LSR   	A          		; SHIFT INTO CARRY
		LSR   	A
		LSR   	A 
		LSR   	A 
		ROL   	RANDNO+1   		; SHIFT RANDNO LEFT ONE BRINGING IN CARRY
		ROL   	RANDNO
		DEY              		; TEST IF 8 NEW RANDOM BITS COMPUTED 
		BNE   	RAND1      		; LOOP FOR MORE IF NOT
		RTS              		; RETURN
 
;       EXPONENTIALLY DISTRIBUTED RANDOM NUMBER SUBROUTINE
;       RULES OF USE SAME AS RAND, 8 BIT RESULT RETURNED IN A
;       AN EXPONENTIAL DISTRIBUTION MEANS THAT THE PROBABILITY OF A 
;       RESULT BETWEEN 10 AND 20 IS THE SAME AS THE PROBABILITY OF A 
;       RESULT BETWEEN 100 AND 200.
;       NOTE THAT THE PROBABILITY OF A ZERO RESULT IS ZERO.
 
RNDEXP JSR   	RAND       		; GET TWO NEW RANDOM BYTES
       JSR   	RAND
       LDA   	RANDNO     		; CONVERT ONE OF THE BYTES TO A RANDOM
       AND   	#7         		; VALUE BETWEEN 0 AND 7 AND PUT IN Y AS A
       TAY              		; SHIFT COUNT
       INY 
       LDA   	RANDNO+1   		; GET THE OTHER RANDOM NUMBER AND SHIFT IT
RNDXP1 DEY              		; RIGHT ACCORDING TO Y
       BEQ   	RNDXP2
       LSR	 	A
       JMP   	RNDXP1
RNDXP2 ORA   	#0         		; TEST FOR A ZERO RESULT
       BEQ   	RNDEXP     		; PROHIBIT ZERO RESULTS
       RTS              		; RETURN
 
;       RECTANGLE SIZE TEST SUBROUTINE
;       FORCES LOW VMNO-1 BITS OF A TO BE ZERO AND TESTS FOR ZERO 
;       USES A AND X
 
CKSZAD 	LDX		VMNO       		; GET NUMBER OF VM'S IN USE
		AND   	SZADTB-1,X 		; MASK OUT APPROPRIATE NUMBER OF LOW BITS
		RTS              		; RETURN

SZADTB	.BYTE 	$FF,$FE,$FC,$F8

    	.END     